@@ -123,7 +123,13 @@ def main(argv):
     if args.func.__name__ == "run":  # calling a pipeline independently of `process`
         if not args.tool_path:
             args.tool_path = os.path.join(os.path.expanduser("~"), args.tool_set)
-        execp = "{}_submit_mjobs.py".format(args.tool_set)
+        execp_key = {
+            "sharp": "./submit.sh",
+            "sera": "sera_submit_mjobs.py",
+            "scata": "scata_submit_mjobs.py",
+            "seqc": "seqc_submit_mjobs.py",
+        }
+        execp = execp_key[args.tool_set]
         path = args.tool_path
         args.func(execp, path, **get_params(args,))
     else:
