@@ -7,9 +7,9 @@ destination, and grant access to collaborators to their data.
 """
 import re
 import os
-import random
 import string
 import urllib.parse
+from random import choices, shuffle
 from typing import Tuple
 
 import pandas as pd
@@ -43,9 +43,15 @@ def make_credentials(
         `None`
     """
     link = "https://583643567512.signin.aws.amazon.com/console"
-    password = "".join(
-        random.choice(string.ascii_letters + string.digits) for i in range(8)
+    password = (
+        choices(string.ascii_lowercase)
+        + choices(string.ascii_uppercase)
+        + choices(string.digits)
+        + choices("!@#$%^&*()_+-=[]{}|'")
+        + choices(string.ascii_letters + string.digits + "!@#$%^&*()_+-=[]{}|'", k=4)
     )
+    shuffle(password)
+    password = "".join(password)
     iam_ = boto3.client("iam")
     iam_.create_user(
         UserName=user,
@@ -138,7 +144,9 @@ def get_glob(
     return collect_cmds, new_dest
 
 
-def df_compiler(d: list, m: str = "all", dist: str = "") -> list:
+def df_compiler(d: list, m: str = "all", dist: str = "") \
+        -> \
+        list:
     """\
     Collect essential information for transferring samples to the destination folder
     on AWS.
@@ -146,7 +154,7 @@ def df_compiler(d: list, m: str = "all", dist: str = "") -> list:
     :param d:
         Sample ids
     :param m:
-        Mode of operation, one of `all`, or `hashtags`
+        Mode of operation, one of `all`, or `hashtags`, or `TCR`
     :param dist:
         Target folder destination for transfer, which is the parent directory where
         projects and samples are stored, for a particular sample.
@@ -195,10 +203,30 @@ def df_compiler(d: list, m: str = "all", dist: str = "") -> list:
             "ON s.id=g.scTech_id "
             "WHERE a.id='{}'"
         ),
+        "tcr": (
+            "SELECT a.id, d.Sample, a.request_id, "
+            "a.projectData_id, b.projectShortName AS project, "
+            "a.processed, c.PI_email, "
+            "d.AWS_storage, '{}' AS destination, "
+            "'TCR_lib' AS `table`, "
+            "s.Run_Name "
+            "FROM sample_data AS a "
+            "JOIN project_data AS b "
+            "ON a.projectData_id=b.id "
+            "JOIN lab AS c "
+            "ON b.lab_id=c.id "
+            "JOIN TCR_lib AS d "
+            "ON d.sampleData_id=a.id "
+            "JOIN genome_index AS g "
+            "ON g.id=d.genomeIndex_id "
+            "JOIN sc_tech AS s "
+            "ON s.id=g.scTech_id "
+            "WHERE a.id='{}'"
+        ),
     }
     statements = []
     for i in d:
-        statements.append(statement[m].format(dist, i))
+        statements.append(statement[m.lower()].format(dist, i))
 
     return statements
 
@@ -224,7 +252,7 @@ def data_transfer(
         `s3uri` is the root path for `s3uri/<project_name>/<sample_name>/` where
         `<project_name>/<sample_name>/` are omitted.
     :param mode:
-        Choose between `all` or `hashtags`
+        Choose between `all` or `hashtags`, or `TCR`
 
     :return:
         `None`
@@ -242,7 +270,7 @@ def data_transfer(
         "H2": "Hashtag_results",
         "H3": "Hashtag_results",
         "H4": "Hashtag_results",
-        "VDJ": "VDJ_results",
+        "VDJ": "CR-results",
     }
 
     # read in results_output.csv file with important information
@@ -348,7 +376,7 @@ def data_transfer(
     for i in df.itertuples():
         try:
             # update location in database
-            assert i.Run_Name not in ["H2", "H3", "H4"]
+            assert i.Run_Name not in ["H2", "H3", "H4", "VDJ"]
             statement1 = "UPDATE {} SET AWS_storage='{}' WHERE id={}".format(
                 i.table.split()[0], os.path.join(i.dest, ""), i.id
             )
@@ -370,22 +398,34 @@ def data_transfer(
                 )
                 statement2 = ""
         except AssertionError:
-            if i.status == "analyzed":
+            if i.table == "TCR_lib":
                 statement1 = (
-                    "UPDATE {} SET AWS_storage='{}', "
-                    "analysis_storage='{}', status=5 "
-                    "WHERE sampleData_id={}".format(
-                        i.table,
-                        os.path.join(i.dest, ""),
+                    "UPDATE {} SET AWS_storage='{}', analysis_storage='{}' WHERE "
+                    "sampleData_id={}".format(
+                        i.table, os.path.join(i.dest, ""),
                         os.path.join(i.dest, analysis_results_key[i.Run_Name], ""),
                         i.id,
                     )
                 )
             else:
-                statement1 = (
-                    "UPDATE {} SET AWS_storage='{}', status=5 WHERE "
-                    "sampleData_id={}".format(i.table, os.path.join(i.dest, ""), i.id,)
-                )
+                if i.status == "analyzed":
+                    statement1 = (
+                        "UPDATE {} SET AWS_storage='{}', "
+                        "analysis_storage='{}', status=5 "
+                        "WHERE sampleData_id={}".format(
+                            i.table,
+                            os.path.join(i.dest, ""),
+                            os.path.join(i.dest, analysis_results_key[i.Run_Name], ""),
+                            i.id,
+                        )
+                    )
+                else:
+                    statement1 = (
+                        "UPDATE {} SET AWS_storage='{}', status=5 WHERE "
+                        "sampleData_id={}".format(
+                            i.table, os.path.join(i.dest, ""), i.id,
+                        )
+                    )
             statement2 = ""
         statements = "; ".join([statement1, statement2])
         for com in i.command:
